@@ -5,6 +5,7 @@
 #include <string.h>
 #include <stdlib.h>
 #include <pwd.h>
+#include <errno.h>
 
 #include "mt.h"
 #include "error.h"
@@ -21,14 +22,18 @@ extern int nfd;
 extern int max_x, max_y;
 extern int min_n_bufferlines;
 
-int find_string(int window, char *find, int offset)
+int find_string(int window, char *find, int offset, char **error)
 {
-	int loop, index = -1;
-        regex_t regex;
+	int loop, index = -1, rc;
+	regex_t regex;
 
 	/* compile the searchstring (which can be a regular expression) */
-	if (regcomp(&regex, find, REG_EXTENDED))
+	if ((rc = regcomp(&regex, find, REG_EXTENDED)))
+	{
+		*error = convert_regexp_error(rc, &regex);
+
 		return -1;	/* failed -> not found */
+	}
 
 	for(loop=offset; loop<lb[window].curpos; loop++)
 	{
@@ -39,7 +44,7 @@ int find_string(int window, char *find, int offset)
 		}
 	}
 
-       	regfree(&regex);
+	regfree(&regex);
 
 	return index;
 }
@@ -67,18 +72,21 @@ void scrollback_savefile(int window)
 					char display;
 					char *error;
 					int dummy = -1;
+					regmatch_t *pmatch = NULL;
 
 					/* check filter */
-					(void)check_filter(lb[window].pi[loop], lb[window].Blines[loop], NULL, &error, &dummy, 0, &display);
+					(void)check_filter(lb[window].pi[loop], lb[window].Blines[loop], &pmatch, &error, &dummy, 0, &display);
 					if (error)
 					{
-						fprintf(fh, "---> Error: %s\n", error);
+						fprintf(fh, "---> Error executing regular expression: %s\n", error);
 						myfree(error);
 					}
 					if (display)
 					{
-						fprintf(fh, "%s", lb[window].Blines[loop]);
+						fprintf(fh, "%s\n", lb[window].Blines[loop]?lb[window].Blines[loop]:"");
 					}
+
+					if (pmatch) myfree(pmatch);
 				}
 			}
 
@@ -86,10 +94,7 @@ void scrollback_savefile(int window)
 		}
 		else
 		{
-			mvwprintw(mywin -> win, 6, 2, "Cannot write to file!");
-			mydoupdate();
-
-			wait_for_keypress(-1, 0, mywin, 0);
+			error_popup("Save scrollback buffer", -1, "Cannot write to file, reason: %s", strerror(errno));
 		}
 	}
 
@@ -146,9 +151,6 @@ void scrollback_displayline(NEWWIN *win, int window, int offset, int terminal_of
 void scrollback(void)
 {
 	int window = 0;
-	NEWWIN *mywin1 = create_popup(7, 22);
-
-	win_header(mywin1, "Scrollback");
 
 	if (nfd > 1)
 	{
@@ -158,20 +160,14 @@ void scrollback(void)
 	if (window != -1)
 	{
 		if (lb[window].markset == 0)
-		{
-			mvwprintw(mywin1 -> win, 3, 2, "Cannot scrollback:");
-			mvwprintw(mywin1 -> win, 4, 2, "buffering is disabled!");
-			mydoupdate();
-			wait_for_keypress(HELP_SCROLLBACK_NO_MARK, 0, mywin1, 0);
-		}
+			error_popup("Scrollback", HELP_SCROLLBACK_NO_MARK, "Cannot scrollback: buffering is disabled.");
 	}
 
-	delete_popup(mywin1);
-
 	if (window != -1 && lb[window].markset != 0)
 	{
 		char *find = NULL;
-		NEWWIN *mywin2;
+		int find_ok = 0;
+		NEWWIN *mywin1, *mywin2;
 		int nlines = max_y - 6, ncols = max_x - 6;
 		int offset = max(0, lb[window].curpos - nlines);
 		char redraw = 2;
@@ -235,7 +231,7 @@ void scrollback(void)
 
 				offset--;
 
-                                wmove(mywin2 -> win, 0, 0);
+				wmove(mywin2 -> win, 0, 0);
 				n_to_move = line_lengths[offset];
 				winsdelln(mywin2 -> win, n_to_move);
 
@@ -251,7 +247,7 @@ void scrollback(void)
 					offset++;
 					dummy += line_lengths[offset];
 					if ((dummy > nlines && c == KEY_NPAGE) ||
-					    (dummy > (nlines/4) && c == KEY_DOWN))
+							(dummy > (nlines/4) && c == KEY_DOWN))
 					{
 						offset--;
 						break;
@@ -286,7 +282,7 @@ void scrollback(void)
 #endif
 			else if (toupper(c) == 'F' || c == '/')
 			{
-				char *dummy;
+				char *dummy, *error;
 				int new_f_index;
 				NEWWIN *mywin = create_popup(5, 40);
 
@@ -298,19 +294,32 @@ void scrollback(void)
 
 				redraw = 2; /* force redraw */
 
-				new_f_index = find_string(window, find, 0);
+				new_f_index = find_string(window, find, 0, &error);
 				if (new_f_index == -1)
-					wrong_key();
+				{
+					if (error)
+					{
+						error_popup("Error in regular expression", -1, "Error string: %s", error);
+						myfree(error);
+					}
+					else
+						wrong_key();
+
+					find_ok = 0;
+				}
 				else
+				{
+					find_ok = 1;
 					offset = new_f_index;
+				}
 
 				delete_popup(mywin);
 			}
 			else if (toupper(c) == 'N')
 			{
-				if (find)
+				if (find != NULL && find_ok != 0)
 				{
-					int new_f_index = find_string(window, find, offset + 1);
+					int new_f_index = find_string(window, find, offset + 1, NULL);
 					if (new_f_index == -1)
 						wrong_key();
 					else
@@ -363,9 +372,7 @@ void delete_mark(void)
 	{
 		if (lb[window].markset == 0)
 		{
-			mvwprintw(mywin -> win, 3, 2, "No mark was set!");
-			mydoupdate();
-			wait_for_keypress(HELP_SCROLLBACK_DELETE_MARK_NO_MARK, 0, mywin, 0);
+			error_popup("Delete mark", HELP_SCROLLBACK_DELETE_MARK_NO_MARK, "No mark was set!");
 		}
 		else
 		{
@@ -375,6 +382,7 @@ void delete_mark(void)
 			delete_array(lb[window].Blines, lb[window].curpos);
 			lb[window].Blines = NULL;
 			myfree(lb[window].pi);
+			lb[window].pi = NULL;
 
 			lb[window].curpos = 0;
 		}
