@@ -126,12 +126,12 @@ void * myrealloc(void *oldp, int newsize, char *what)
 	add_mem_element(dummy, newsize, what);
 	signal(SIGHUP, dump_mem);
 #else
-/* ----------------------------------------------------
- * add code for repeatingly retry? -> then configurable
- * via configurationfile with number of retries and/or
- * sleep
- * ----------------------------------------------------
- */
+	/* ----------------------------------------------------
+	 * add code for repeatingly retry? -> then configurable
+	 * via configurationfile with number of retries and/or
+	 * sleep
+	 * ----------------------------------------------------
+	 */
 	void *dummy = realloc(oldp, newsize);
 	if (!dummy)
 		error_exit("failed to reallocate to %d bytes for %s\n", newsize, what);
@@ -156,49 +156,6 @@ char * mystrdup(char *in, char *what)
 	return dummy;
 }
 
-int resize(void **pnt, int n, int *len, int size)
-{
-        if (n == *len)
-        {
-                *len = (*len) ? (*len) * 2 : 4096;
-                *pnt = myrealloc(*pnt, (*len) * size, "resize()");
-        }
-	else if (n > *len || n<0 || *len<0)
-		error_exit("resize: fatal memory corruption problem: n > len || n<0 || len<0!\n");
-
-	return 0;
-}
-
-ssize_t READ(int fd, char *whereto, size_t len)
-{
-	ssize_t cnt=0;
-
-	while(len>0)
-	{
-		ssize_t rc;
-
-		rc = read(fd, whereto, len);
-
-		if (rc == -1)
-		{
-			if (errno != EINTR && errno != EAGAIN)
-				error_exit("Problem reading from file\n");
-		}
-		else if (rc == 0)
-		{
-			break;
-		}
-		else
-		{
-			whereto += rc;
-			len -= rc;
-			cnt += rc;
-		}
-	}
-
-	return cnt;
-}
-
 ssize_t WRITE(int fd, char *whereto, size_t len)
 {
 	ssize_t cnt=0;
@@ -229,49 +186,6 @@ ssize_t WRITE(int fd, char *whereto, size_t len)
 	return cnt;
 }
 
-char * read_line_fd(int fd)
-{
-	char *str = mymalloc(80, "read_line_fd buffer");
-	int n_in=0, size=80;
-
-	for(;;)
-	{
-		char c;
-		int rc;
-
-		/* read one character */
-		if ((rc = READ(fd, &c, 1)) != 1)
-		{
-			if (n_in == 0)
-			{
-				myfree(str);
-				str = NULL;
-			}
-			return str;
-		}
-
-		/* resize input-buffer */
-		resize((void **)&str, n_in, &size, sizeof(char));
-
-		/* EOF or \n == end of line */
-		if (c == 10 || c == EOF)
-			break;
-
-		/* ignore CR */
-		if (c == 13)
-			continue;
-
-		/* add to string */
-		str[n_in++] = c;
-	}
-
-	/* terminate string */
-	resize((void **)&str, n_in, &size, sizeof(char));
-	str[n_in] = 0x00;
-
-	return str;
-}
-
 /** get_load
  * - in:      nothing
  * - returns: char *
@@ -281,9 +195,9 @@ char * read_line_fd(int fd)
  */
 char *get_load(void)
 {
-#if defined(__FreeBSD__) || defined(linux) || defined(__OpenBSD__) || defined(__NetBSD__) || defined(__APPLE__) || defined(sun)
+#if !defined(__UCLIBC__) && (defined(__FreeBSD__) || defined(linux) || defined(__OpenBSD__) || defined(__NetBSD__) || defined(__APPLE__) || defined(sun))
 	double loadavg[3];
-	char *str = (char *)mymalloc(20, "loadavg string");
+	char *str = (char *)mymalloc(LOADAVG_STR_LEN, "loadavg string");
 
 #if defined(__GLIBC__) && ( __GLIBC__ < 2 || (__GLIBC__ == 2 && __GLIBC_MINOR__ < 2))
 	/* Older glibc doesn't have getloadavg() - use sysinfo() */
@@ -310,7 +224,7 @@ char *get_load(void)
 		error_exit("getloadavg() failed");
 	}
 #endif
-	sprintf(str, "%1.2f %1.2f %1.2f", loadavg[0], loadavg[1], loadavg[2]);
+	snprintf(str, LOADAVG_STR_LEN, "%1.2f %1.2f %1.2f", loadavg[0], loadavg[1], loadavg[2]);
 
 	return str;
 #else
@@ -320,11 +234,11 @@ char *get_load(void)
 
 int get_vmsize(pid_t pid)
 {
+	int vmsize = -1;
+#if defined(linux)
 	FILE *fh;
 	char *path = mymalloc(path_max, "path");
-	int vmsize = -1;
 
-#if defined(linux)
 	sprintf(path, "/proc/%d/stat", pid);
 
 	fh = fopen(path, "r");
@@ -340,9 +254,9 @@ int get_vmsize(pid_t pid)
 
 		myfree(dummystr);
 	}
-#endif
 
 	myfree(path);
+#endif
 
 	return vmsize;
 }
@@ -410,15 +324,11 @@ void delete_array(char **list, int n)
 
 int find_char_offset(char *str, char what)
 {
-	int loop, len = strlen(str);
+	char *pnt = strchr(str, what);
+	if (!pnt)
+		return -1;
 
-	for(loop=0; loop<len; loop++)
-	{
-		if (str[loop] == what)
-			return loop;
-	}
-
-	return -1;
+	return (int)(pnt - str);
 }
 
 int file_info(char *filename, off64_t *file_size, time_t *ts)
@@ -440,6 +350,13 @@ int file_info(char *filename, off64_t *file_size, time_t *ts)
 	return 0;
 }
 
+int file_exist(char *filename)
+{
+	struct stat64 buf;
+
+	return stat64(filename, &buf);
+}
+
 char * convert_regexp_error(int error, const regex_t *preg)
 {
 	/* errors are specified not to be longer then 256 characters */
@@ -476,22 +393,18 @@ char check_date(void)
 	return 0;
 }
 
-#define M_KB (1024)
-#define M_MB (M_KB * 1024)
-#define M_GB (M_MB * 1024)
-
-char * amount_to_str(int amount)
+char * amount_to_str(long long int amount)
 {
-	char *out = mymalloc(3 + 2 + 1, "converted amount string");	/* ...XB\0 */
+	char *out = mymalloc(AMOUNT_STR_LEN, "converted amount string");	/* ...XB\0 */
 
-	if (amount > M_GB)	/* GB */
-		sprintf(out, "%dGB", (amount + M_GB - 1) / M_GB);
-	else if (amount > M_MB)	/* MB */
-		sprintf(out, "%dMB", (amount + M_MB - 1) / M_MB);
-	else if (amount > M_KB)	/* KB */
-		sprintf(out, "%dKB", (amount + M_KB - 1) / M_KB);
+	if (amount >= M_GB)	/* GB */
+		snprintf(out, AMOUNT_STR_LEN, "%dGB", (int)((amount + M_GB - 1) / M_GB));
+	else if (amount >= M_MB)	/* MB */
+		snprintf(out, AMOUNT_STR_LEN, "%dMB", (int)((amount + M_MB - 1) / M_MB));
+	else if (amount >= M_KB)	/* KB */
+		snprintf(out, AMOUNT_STR_LEN, "%dKB", (int)((amount + M_KB - 1) / M_KB));
 	else
-		sprintf(out, "%d", amount);
+		snprintf(out, AMOUNT_STR_LEN, "%d", (int)(amount));
 
 	return out;
 }
@@ -515,7 +428,6 @@ char * replace_string(char *in, int pos_start, int pos_end, char *with_what)
 struct passwd *getuserinfo(void)
 {
 	struct passwd *pp = getpwuid(geteuid());
-
 	if (!pp)
 		error_exit("failed to get passwdstructure for effective user id %d", geteuid());
 
@@ -532,7 +444,7 @@ int myopen(char *path, int mode)
 		fd = open64(path, mode);
 		if (fd == -1)
 		{
-			if (errno == EINTR)
+			if (errno == EINTR || errno == EAGAIN)
 				continue;
 
 			return fd;
@@ -550,7 +462,7 @@ int myclose(int fd)
 	{
 		if (close(fd) == -1)
 		{
-			if (errno == EINTR)
+			if (errno == EINTR || errno == EAGAIN)
 				continue;
 
 			return -1;
